import { format } from 'prettier/standalone';
import React, { useCallback, useMemo } from 'react';
import { PrismAsyncLight } from 'react-syntax-highlighter';
import { oneDark, oneLight } from 'react-syntax-highlighter/dist/esm/styles/prism';
import parserBabel from 'prettier/parser-babel';
import parserHtml from 'prettier/parser-html';
import parserPostcss from 'prettier/parser-postcss';
import parserMarkdown from 'prettier/parser-markdown';
import parserGraphql from 'prettier/parser-graphql';
import parserYaml from 'prettier/parser-yaml';
import styled from 'styled-components';

var CodeHighlighterTheme;
(function (CodeHighlighterTheme) {
  CodeHighlighterTheme["Light"] = "light";
  CodeHighlighterTheme["Dark"] = "dark";
})(CodeHighlighterTheme || (CodeHighlighterTheme = {}));

const BABEL_LANGUAGES = ['tsx', 'jsx', 'typescript', 'javascript'];
const HTML_LANGUAGES = ['html'];
const CSS_LANGUAGES = ['css', 'less', 'scss'];
const MARKDOWN_LANGUAGES = ['markdown'];
const GRAPHQL_LANGUAGES = ['graphql'];
const YAML_LANGUAGES = ['yaml'];

// JavaScript/TypeScript
const getParserForLanguage = language => {
  let parser;
  let plugin;
  switch (true) {
    case BABEL_LANGUAGES.includes(language):
      parser = 'babel-ts';
      plugin = parserBabel;
      break;
    case HTML_LANGUAGES.includes(language):
      parser = 'html';
      plugin = parserHtml;
      break;
    case CSS_LANGUAGES.includes(language):
      parser = 'css';
      plugin = parserPostcss;
      break;
    case MARKDOWN_LANGUAGES.includes(language):
      parser = 'markdown';
      plugin = parserMarkdown;
      break;
    case GRAPHQL_LANGUAGES.includes(language):
      parser = 'graphql';
      plugin = parserGraphql;
      break;
    case YAML_LANGUAGES.includes(language):
      parser = 'yaml';
      plugin = parserYaml;
      break;
  }
  if (parser && plugin) {
    return {
      trailingComma: 'all',
      bracketSpacing: true,
      singleQuote: true,
      jsxSingleQuote: true,
      bracketSameLine: false,
      arrowParens: 'always',
      proseWrap: 'always',
      parser,
      plugins: [plugin]
    };
  }
  return undefined;
};
const formatLanguage = language => {
  if (!language) {
    return '';
  }
  const formattedLanguage = {
    abap: 'ABAP',
    abnf: 'ABNF',
    actionscript: 'ActionScript',
    ada: 'Ada',
    agda: 'Agda',
    al: 'AL',
    antlr4: 'ANTLR4',
    apacheconf: 'ApacheConf',
    apex: 'Apex',
    apl: 'APL',
    applescript: 'AppleScript',
    aql: 'AQL',
    arduino: 'Arduino',
    arff: 'ARFF',
    asciidoc: 'AsciiDoc',
    asm6502: 'ASM6502',
    asmatmel: 'ASMatmel',
    aspnet: 'ASP.NET',
    autohotkey: 'AutoHotKey',
    autoit: 'AutoIt',
    avisynth: 'AviSynth',
    avroIdl: 'Avro IDL',
    bash: 'Bash',
    basic: 'Basic',
    batch: 'Batch',
    bbcode: 'BBCode',
    bicep: 'Bicep',
    birb: 'Birb',
    bison: 'Bison',
    bnf: 'BNF',
    brainfuck: 'Brainfuck',
    brightscript: 'BrightScript',
    bro: 'Bro',
    bsl: 'BSL',
    c: 'C',
    cfscript: 'CFScript',
    chaiscript: 'ChaiScript',
    cil: 'CIL',
    clike: 'C-like',
    clojure: 'Clojure',
    cmake: 'CMake',
    cobol: 'COBOL',
    coffeescript: 'CoffeeScript',
    concurnas: 'Concurnas',
    coq: 'Coq',
    cpp: 'C++',
    crystal: 'Crystal',
    csharp: 'C#',
    cshtml: 'CSHTML',
    csp: 'CSP',
    cssExtras: 'CSS Extras',
    css: 'CSS',
    csv: 'CSV',
    cypher: 'Cypher',
    d: 'D',
    dart: 'Dart',
    dataweave: 'DataWeave',
    dax: 'DAX',
    dhall: 'Dhall',
    diff: 'Diff',
    django: 'Django',
    dnsZoneFile: 'DNS Zone File',
    docker: 'Docker',
    dot: 'DOT',
    ebnf: 'EBNF',
    editorconfig: 'EditorConfig',
    eiffel: 'Eiffel',
    ejs: 'EJS',
    elixir: 'Elixir',
    elm: 'Elm',
    erb: 'ERB',
    erlang: 'Erlang',
    etlua: 'ETLua',
    excelFormula: 'Excel Formula',
    factor: 'Factor',
    falselang: 'Falselang',
    firestoreSecurityRules: 'Firestore Security Rules',
    flow: 'Flow',
    fortran: 'Fortran',
    fsharp: 'F#',
    ftl: 'FTL',
    gap: 'GAP',
    gcode: 'G-code',
    gdscript: 'GDScript',
    gedcom: 'GEDCOM',
    gherkin: 'Gherkin',
    git: 'Git',
    glsl: 'GLSL',
    gml: 'GML',
    gn: 'GN',
    goModule: 'Go Module',
    go: 'Go',
    graphql: 'GraphQL',
    groovy: 'Groovy',
    haml: 'Haml',
    handlebars: 'Handlebars',
    haskell: 'Haskell',
    haxe: 'Haxe',
    hcl: 'HCL',
    hlsl: 'HLSL',
    hoon: 'Hoon',
    hpkp: 'HPKP',
    hsts: 'HSTS',
    html: 'HTML',
    http: 'HTTP',
    ichigojam: 'IchigoJam',
    icon: 'Icon',
    icuMessageFormat: 'ICU Message Format',
    idris: 'Idris',
    iecst: 'IECST',
    ignore: 'Ignore',
    inform7: 'Inform7',
    ini: 'INI',
    io: 'Io',
    j: 'J',
    java: 'Java',
    javadoc: 'Javadoc',
    javadoclike: 'Javadoc-like',
    javascript: 'JavaScript',
    javastacktrace: 'Java Stack Trace',
    jexl: 'JEXL',
    jolie: 'Jolie',
    jq: 'jq',
    jsExtras: 'JS Extras',
    jsTemplates: 'JS Templates',
    jsdoc: 'JSDoc',
    json: 'JSON',
    json5: 'JSON5',
    jsonp: 'JSONP',
    jsstacktrace: 'JS Stack Trace',
    jsx: 'JSX',
    julia: 'Julia',
    keepalived: 'Keepalived',
    keyman: 'Keyman',
    kotlin: 'Kotlin',
    kumir: 'KuMir',
    kusto: 'Kusto',
    latex: 'LaTeX',
    latte: 'Latte',
    less: 'Less',
    lilypond: 'LilyPond',
    liquid: 'Liquid',
    lisp: 'Lisp',
    livescript: 'LiveScript',
    llvm: 'LLVM',
    log: 'Log',
    lolcode: 'LOLCODE',
    lua: 'Lua',
    magma: 'Magma',
    makefile: 'Makefile',
    markdown: 'Markdown',
    markupTemplating: 'Markup Templating',
    markup: 'Markup',
    matlab: 'MATLAB',
    maxscript: 'MAXScript',
    mel: 'MEL',
    mermaid: 'Mermaid',
    mizar: 'Mizar',
    mongodb: 'MongoDB',
    monkey: 'Monkey',
    moonscript: 'MoonScript',
    n1ql: 'N1QL',
    n4js: 'N4JS',
    nand2tetrisHdl: 'Nand2Tetris HDL',
    naniscript: 'NaninScript',
    nasm: 'NASM',
    neon: 'NEON',
    nevod: 'NevoD',
    nginx: 'Nginx',
    nim: 'Nim',
    nix: 'Nix',
    nsis: 'NSIS',
    objectivec: 'Objective-C',
    ocaml: 'OCaml',
    opencl: 'OpenCL',
    openqasm: 'OpenQasm',
    oz: 'Oz',
    parigp: 'PARI/GP',
    parser: 'Parser',
    pascal: 'Pascal',
    pascaligo: 'Pascaligo',
    pcaxis: 'PC-Axis',
    peoplecode: 'PeopleCode',
    perl: 'Perl',
    phpExtras: 'PHP Extras',
    php: 'PHP',
    phpdoc: 'PHPDoc',
    plsql: 'PL/SQL',
    powerquery: 'PowerQuery',
    powershell: 'PowerShell',
    processing: 'Processing',
    prolog: 'Prolog',
    promql: 'PromQL',
    properties: 'Properties',
    protobuf: 'Protocol Buffers',
    psl: 'PSL',
    pug: 'Pug',
    puppet: 'Puppet',
    pure: 'Pure',
    purebasic: 'PureBasic',
    purescript: 'PureScript',
    python: 'Python',
    q: 'Q',
    qml: 'QML',
    qore: 'Qore',
    qsharp: 'Q#',
    r: 'R',
    racket: 'Racket',
    reason: 'Reason',
    regex: 'RegExp',
    rego: 'Rego',
    renpy: 'RenPy',
    rest: 'reST',
    rip: 'Rip',
    roboconf: 'Roboconf',
    robotframework: 'Robot Framework',
    ruby: 'Ruby',
    rust: 'Rust',
    sas: 'SAS',
    sass: 'Sass',
    scala: 'Scala',
    scheme: 'Scheme',
    scss: 'SCSS',
    shellSession: 'Shell Session',
    smali: 'Smali',
    smalltalk: 'Smalltalk',
    smarty: 'Smarty',
    sml: 'SML',
    solidity: 'Solidity',
    solutionFile: 'Solution File',
    soy: 'Soy',
    sparql: 'SPARQL',
    splunkSpl: 'Splunk SPL',
    sqf: 'SQF',
    sql: 'SQL',
    squirrel: 'Squirrel',
    stan: 'Stan',
    stylus: 'Stylus',
    swift: 'Swift',
    systemd: 'systemd',
    t4Cs: 'T4CS',
    t4Templating: 'T4 Templating',
    t4Vb: 'T4VB',
    tap: 'TAP',
    tcl: 'Tcl',
    textile: 'Textile',
    toml: 'TOML',
    tremor: 'Tremor',
    tsx: 'TSX',
    tt2: 'TT2',
    turtle: 'Turtle',
    twig: 'Twig',
    typescript: 'TypeScript',
    typoscript: 'TypoScript',
    unrealscript: 'UnrealScript',
    uorazor: 'Uorazor',
    uri: 'URI',
    v: 'V',
    vala: 'Vala',
    vbnet: 'VB.NET',
    velocity: 'Velocity',
    verilog: 'Verilog',
    vhdl: 'VHDL',
    vim: 'Vim',
    visualBasic: 'Visual Basic',
    warpscript: 'WarpScript',
    wasm: 'WebAssembly',
    webIdl: 'WebIDL',
    wiki: 'Wiki',
    wolfram: 'Wolfram',
    wren: 'Wren',
    xeora: 'Xeora',
    xmlDoc: 'XML Doc',
    xojo: 'Xojo',
    xquery: 'XQuery',
    yaml: 'YAML',
    yang: 'YANG',
    zig: 'Zig'
  };
  return formattedLanguage[language] || language;
};

const StyledCodeHighlighter = styled.div`
    margin: 4px 0;
    background-color: ${_ref => {
  let {
    codeTheme
  } = _ref;
  return codeTheme === CodeHighlighterTheme.Dark ? '#282c34' : '#fafafa';
}};
    border-radius: 8px;
    padding-bottom: 6px;

    pre {
        margin: 0 !important;
        overflow: auto;
        padding: 1em;
        line-height: 1.5;

        // Styles for custom scrollbar

        &::-webkit-scrollbar {
            height: 5px;
        }

        &::-webkit-scrollbar-track {
            background-color: transparent;
        }

        &::-webkit-scrollbar-button {
            background-color: transparent;
            height: 2px;
        }

        &::-webkit-scrollbar-thumb {

            background-color: rgba(${_ref2 => {
  let {
    codeTheme
  } = _ref2;
  return codeTheme === CodeHighlighterTheme.Dark ? '229, 229, 229' : '153, 153, 153';
}},
            1);
            border-radius: 20px;
        }

        // Scrollbar styles for Firefox. The above styles are not supported in Firefox, these styles are
        // only supported in Firefox:

        * {
            scrollbar-color: rgba(${_ref3 => {
  let {
    codeTheme
  } = _ref3;
  return codeTheme === CodeHighlighterTheme.Dark ? '229, 229, 229' : '153, 153, 153';
}},
            1);
        ) transparent;
            scrollbar-width: thin;
        }
    }
`;
const StyledCodeHighlighterHeader = styled.div`
    display: flex;
    align-items: center;
    justify-content: space-between;
    border-bottom: 1px solid
        ${_ref4 => {
  let {
    codeTheme
  } = _ref4;
  return codeTheme === CodeHighlighterTheme.Dark ? '#e5e5e5' : '#999999';
}};
    padding: 8px 12px;
`;
const StyledCodeHighlighterFileName = styled.span`
    color: ${_ref5 => {
  let {
    codeTheme
  } = _ref5;
  return codeTheme === CodeHighlighterTheme.Dark ? '#e5e5e5' : '#999999';
}};
    font-family: sans-serif;
    font-size: small;
`;

const StyledCopyToClipboard = styled.div`
    display: flex;
    align-items: center;
`;
const StyledCopyToClipboardIcon = styled.svg`
    cursor: pointer;
`;
const StyledCopyToClipboardText = styled.span`
    color: ${_ref => {
  let {
    codeTheme
  } = _ref;
  return codeTheme === CodeHighlighterTheme.Dark ? '#e5e5e5' : '#999999';
}};
    font-size: small;
    font-family: sans-serif;
    margin-left: 4px;
    cursor: pointer;
`;

const CopyToClipboard = _ref => {
  let {
    copyButtonText,
    text,
    theme
  } = _ref;
  const handleClick = () => {
    void navigator.clipboard.writeText(text);
  };
  return /*#__PURE__*/React.createElement(StyledCopyToClipboard, {
    onClick: handleClick
  }, /*#__PURE__*/React.createElement(StyledCopyToClipboardIcon, {
    xmlns: "http://www.w3.org/2000/svg",
    height: "16",
    width: "14",
    viewBox: "0 0 448 512"
  }, /*#__PURE__*/React.createElement("path", {
    fill: theme === CodeHighlighterTheme.Dark ? '#e5e5e5' : '#999999',
    d: "M384 336H192c-8.8 0-16-7.2-16-16V64c0-8.8 7.2-16 16-16l140.1 0L400 115.9V320c0 8.8-7.2 16-16 16zM192 384H384c35.3 0 64-28.7 64-64V115.9c0-12.7-5.1-24.9-14.1-33.9L366.1 14.1c-9-9-21.2-14.1-33.9-14.1H192c-35.3 0-64 28.7-64 64V320c0 35.3 28.7 64 64 64zM64 128c-35.3 0-64 28.7-64 64V448c0 35.3 28.7 64 64 64H256c35.3 0 64-28.7 64-64V416H272v32c0 8.8-7.2 16-16 16H64c-8.8 0-16-7.2-16-16V192c0-8.8 7.2-16 16-16H96V128H64z"
  })), copyButtonText && ( /*#__PURE__*/React.createElement(StyledCopyToClipboardText, {
    codeTheme: theme
  }, copyButtonText)));
};
CopyToClipboard.displayName = 'CopyToClipboard';

const CodeHighlighter = _ref => {
  let {
    theme = CodeHighlighterTheme.Dark,
    code,
    copyButtonText,
    language,
    highlightedLines,
    shouldFormatCode = false,
    shouldShowLineNumbers = false
  } = _ref;
  // function to style highlighted code
  const lineWrapper = useCallback(lineNumber => {
    let style = {
      backgroundColor: 'none',
      display: 'block',
      borderRadius: '2px'
    };
    if (!highlightedLines) {
      return {
        style
      };
    }
    if (highlightedLines.added && highlightedLines.added.includes(lineNumber)) {
      style = {
        ...style,
        backgroundColor: '#2EF29930'
      };
    } else if (highlightedLines.removed && highlightedLines.removed.includes(lineNumber)) {
      style = {
        ...style,
        backgroundColor: '#F22E5B30'
      };
    } else if (highlightedLines.marked && highlightedLines.marked.includes(lineNumber)) {
      style = {
        ...style,
        backgroundColor: '#cccccc40'
      };
    }
    return {
      style
    };
  }, [highlightedLines]);
  const formattedCode = useMemo(() => {
    if (language) {
      const config = getParserForLanguage(language);
      if (shouldFormatCode && config) {
        return format(code, config);
      }
      return code;
    }
    return code;
  }, [code, language, shouldFormatCode]);
  return useMemo(() => ( /*#__PURE__*/React.createElement(StyledCodeHighlighter, {
    codeTheme: theme
  }, /*#__PURE__*/React.createElement(StyledCodeHighlighterHeader, {
    codeTheme: theme
  }, /*#__PURE__*/React.createElement(StyledCodeHighlighterFileName, {
    codeTheme: theme
  }, formatLanguage(language)), /*#__PURE__*/React.createElement(CopyToClipboard, {
    text: code,
    theme: theme,
    copyButtonText: copyButtonText
  })), /*#__PURE__*/React.createElement(PrismAsyncLight, {
    language: language ?? '',
    showLineNumbers: shouldShowLineNumbers,
    style: theme === CodeHighlighterTheme.Dark ? oneDark : oneLight,
    wrapLines: true,
    lineProps: lineWrapper
  }, formattedCode))), [theme, language, code, copyButtonText, shouldShowLineNumbers, lineWrapper, formattedCode]);
};
CodeHighlighter.displayName = 'CodeHighlighter';

export { CodeHighlighter, CodeHighlighterTheme };
//# sourceMappingURL=code-highlighter-react.esm.js.map
